package net.wanji.utc.service.staticinfo.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.dto.CrossSchedulesDTO;
import net.wanji.databus.dto.PlanSectionDTO;
import net.wanji.databus.vo.CrossIdVO;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.cache.UtcFeignClientCache;
import net.wanji.utc.common.Result;
import net.wanji.utc.service.staticinfo.WanJiCommonStaticInfoService;
import net.wanji.utc.util.HttpRestUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @author duanruiming
 * @date 2023/01/09 9:21
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class WanJiCommonStaticInfoServiceImpl implements WanJiCommonStaticInfoService {

    private final HttpRestUtil httpRestUtil;
    @Value("url")
    private String rpcUri;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSectionMapper crossSectionMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;
    private final CrossInfoCache crossInfoCache;
    private final UtcFeignClientCache utcFeignClientCache;


    @Override
    public void schemePhaseLights(String crossId, String signalId) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        CrossIdVO crossIdVO = new CrossIdVO();
        crossIdVO.setCrossId(crossId);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).schemePhaseLights(crossIdVO);
        ObjectMapper objectMapper = new ObjectMapper();
        SchemePhaseLightsVO content = objectMapper.convertValue(jsonViewObject.getContent(), SchemePhaseLightsVO.class);
        List<BaseCrossSchemePO> crossSchemeList = content.getCrossSchemeList();
        crossSchemeList.forEach(crossSchemePO -> {
            crossSchemeMapper.deleteOne(crossId, crossSchemePO.getSchemeNo());
            crossSchemeMapper.insertOne(crossSchemePO);
        });
        List<CrossPhasePO> crossPhaseList = content.getCrossPhaseList();
        crossPhaseList.forEach(crossPhasePO -> {
            crossPhaseMapper.deleteBatch(crossId, crossPhasePO.getPlanId());
            crossPhaseMapper.insertOne(crossPhasePO);
        });

        List<CrossLightsPO> crossLightsList = content.getCrossLightsList();
        crossLightsList.forEach(crossLightsPO -> {
            crossLightsMapper.deleteOne(crossId, crossLightsPO.getName());
            crossLightsMapper.insertOne(crossLightsPO);
        });
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = content.getCrossPhaseLightsPOList();
        if (!CollectionUtils.isEmpty(crossPhaseLightsPOList)) {
            crossPhaseLightsMapper.deleteByCrossId(crossPhaseLightsPOList.get(0).getCrossId());
            crossPhaseLightsMapper.insertBatch(crossPhaseLightsPOList);
        }
    }

    @Override
    public void planSection(String crossId, String signalId, Integer planNo) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        PlanSectionDTO planSectionDTO = new PlanSectionDTO();
        planSectionDTO.setCrossId(crossId);
        planSectionDTO.setPlanNo(planNo);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).planSection(planSectionDTO);
        ObjectMapper mapper = new ObjectMapper();
        List<PlanSectionVO> content = mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<PlanSectionVO>>() {
        });
        if (!CollectionUtils.isEmpty(content)) {
            for (PlanSectionVO planSectionVO : content) {
                List<CrossPlanPO> crossPlanPOList = planSectionVO.getCrossPlanPOList();
                crossPlanPOList.forEach(crossPlanPO -> {
                    crossPlanMapper.deleteOne(crossId, Integer.parseInt(crossPlanPO.getPlanNo()));
                    crossPlanMapper.insertOne(crossPlanPO);
                });

                List<CrossSectionPO> crossSectionPOList = planSectionVO.getCrossSectionPOList();
                crossSectionPOList.forEach(crossSectionPO -> {
                    crossSectionMapper.deleteOne(crossId, Integer.parseInt(crossSectionPO.getSectionNo()), null);
                    crossSectionMapper.insertOne(crossSectionPO);
                });
            }
        }
    }

    @Override
    public Result<List<CrossSchedulesPO>> schedules(String crossId, String signalId) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        CrossSchedulesDTO crossSchedulesDTO = new CrossSchedulesDTO();
        crossSchedulesDTO.setManufacturerCode(manufacturerIdCode);
        crossSchedulesDTO.setCrossIdList(Arrays.asList(crossId));
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).crossSchedules(crossSchedulesDTO);
        ObjectMapper mapper = new ObjectMapper();
        List<CrossSchedulesPO> content = mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<CrossSchedulesPO>>() {
        });

        if (Objects.nonNull(content)) {
            if (!CollectionUtils.isEmpty(content)) {
                content.forEach(crossSchedulesPO -> {
                    crossSchedulesMapper.deleteByCrossId(crossId);
                    crossSchedulesMapper.insertBatch(content);
                });
            }
        }
        return Result.success(content);
    }

    public static Map<String, String> getCommonParamMap(String crossId, String signalId) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("crossId", crossId);
        paramMap.put("signalId", signalId);
        return paramMap;
    }

    @Override
    public JsonViewObject rpcService(HttpRestUtil httpRestUtil, String url, Map<String, String> headers, String param) {
        return WanJiCommonStaticInfoService.super.rpcService(httpRestUtil, url, headers, param);
    }
}
